namespace OptionalProj.Samples.MyModule
{
    public static class Program
    {
        public static void Main()
        {
            // Creating objects: codec and two addresses
            OSS.Asn1rt.IEncoder encoder = OptionalProjFactory.CreateEncoder();
            OSS.Asn1rt.IDecoder decoder = OptionalProjFactory.CreateDecoder();

            OptionalProj.MyModule.Address basicAddr = new OptionalProj.MyModule.Address();
            OptionalProj.MyModule.Address fullAddr = new OptionalProj.MyModule.Address();

            // Assigning the required data 
            // NOTE, the DEFAULT value is implicitly set (by the object constructor) 
            basicAddr.City = "Somerset";   // required
            System.Console.WriteLine(basicAddr);
            // output: value Address ::= { city "Somerset", country "USA" } 

            // 1 Encode without DEFAULT values
            encoder.Encode(basicAddr);
            System.Console.WriteLine(OSS.Asn1rt.Util.ByteArray.ByteArrayToHexString(encoder.Data, " ", 32));
            // The 'country' is not encoded because it matches the DEFAULT
            // output: 30 0a 81 08 53 6f 6d 65 72 73 65 74

            // 2 Encode with DEFAULT values 
            // use encoder property to force defaults to be encoded 
            encoder.SetProperty("encodingDefaultValues", "true");
            encoder.Encode(basicAddr);
            System.Console.WriteLine(OSS.Asn1rt.Util.ByteArray.ByteArrayToHexString(encoder.Data, " ", 32));
            // output: 30 0f 81 08 53 6f 6d 65 72 73 65 74 82 03 55 53 41

            // Decoding it into fullAddr
            decoder.Decode(encoder.Data, fullAddr);
            System.Console.WriteLine(fullAddr);
            // output: value Address ::= { city "Somerset", country "USA" }

            // Filling in the optional 
            if (fullAddr.HasStreet() == false)
                fullAddr.Street = "---";

            // Re-setting to the default
            if (fullAddr.HasCountry() == true)
                basicAddr.HasCountry(false);

            System.Console.WriteLine(fullAddr);
            // Output: value Address ::= { street "---", city "Somerset", country "USA" }
        }
    }
}
