namespace OptionalProj.Samples.MyModule
{
    public static class Program
    {
        public static void Main()
        {
            // Creating objects: codec and two addresses
            OptionalProj.BerCodec codec = new OptionalProj.BerCodec();
            
            OptionalProj.MyModule.Address basicAddr = new OptionalProj.MyModule.Address();
            OptionalProj.MyModule.Address fullAddr = new OptionalProj.MyModule.Address();

            // Initialize, and do not set optional or default values 
            // NOTE, the DEFAULT value is NOT implicitly set (as it was in v3) 
            basicAddr.City = "Somerset";   // required
            Oss.Asn1.ValueNotationFormatter.Print(basicAddr, System.Console.Out);
            // Output: value Address ::= {  city "Somerset" }

            // 1 Encode without DEFAULT values
            byte[] ber = codec.Encode(basicAddr);
            Oss.Asn1.ValueNotationFormatter.Print(ber, ber.Length, System.Console.Out);
            // The 'country' is not encoded because it is a null
            // Output: 300A8108 536F6D65 72736574

            // 2 Encode with DEFAULT values 
            // Explicitly assign the default value to be encoded
            basicAddr.Country = OptionalProj.MyModule.Address.DefaultCountry; 
            ber = codec.Encode(basicAddr);
            Oss.Asn1.ValueNotationFormatter.Print(ber, ber.Length, System.Console.Out);
            // Output: 3300F8108 536F6D65 72736574 82035553 41

            // Decoding it into fullAddr
            codec.Decode(ber, fullAddr);
            Oss.Asn1.ValueNotationFormatter.Print(fullAddr, System.Console.Out);
            // Output: value Address ::= { city "Somerset", country "USA" }

            // Filling in the optional 
            if (fullAddr.Street == null)
                fullAddr.Street = "---";

            // Re-setting to the default
            if (fullAddr.Country == OptionalProj.MyModule.Address.DefaultCountry)
                fullAddr.Country = null;

            // Implicit defaults are not printed
            Oss.Asn1.ValueNotationFormatter.Print(fullAddr, System.Console.Out);
            // Output: value Address ::= { street "---", city "Somerset" }

            // Enable printing implicit defaults
            Oss.Asn1.ValueNotationFormatter.Options.PrintImpliedValues = true;
            Oss.Asn1.ValueNotationFormatter.Print(fullAddr, System.Console.Out);
            // Output: value Address ::= { street "---", city "Somerset",  country "USA" }
        }
    }
}
